#ifndef IMPBRUSH_H
#define IMPBRUSH_H

//
// ImpBrush.h
//
// The header file of virtual brush. All the other brushes inherit from it
//

#include <stdlib.h>

// We have totally six types of brushes (at least). Here they are
#define	BRUSH_POINTS				0
#define BRUSH_LINES					1	
#define BRUSH_CIRCLES				2
#define BRUSH_SCATTERED_POINTS		3
#define BRUSH_SCATTERED_LINES		4
#define BRUSH_SCATTERED_CIRCLES		5
#define NUM_BRUSH_TYPE				6

class ImpressionistDoc;

class Point {
public:
	Point() {};
	Point(int xx, int yy) { x = xx; y = yy; };

	int x, y;
};

class ImpBrush 
{
protected:
	ImpBrush::ImpBrush( ImpressionistDoc* pDoc = NULL, char* name = NULL );

public:
	// The implementation of your brush should realize these virtual functions
	virtual void BrushBegin( Point source, Point target ) {};
	virtual void BrushMove( Point source, Point target ) {};
	virtual void BrushEnd( Point source, Point target ) {};

	void SetColor( Point source );		// according to the source image and the position, determine the draw color

	ImpressionistDoc* GetDocument( void );	// get Doc to communicate with it
	char* BrushName( void );		// return the name of the brush, not used in this version
	
	static int c_nBrushCount;		// how many brushes we have
	static ImpBrush** c_pBrushes;	// and what are they

private:
	ImpressionistDoc* m_pDoc;
	char* m_pBrushName;		// brush's name, not used in this version
};

#endif