// 
// impressionistDoc.cpp
//
// It basically maintain the bitmap for answering the color query from the brush.
// It also acts as the bridge between brushes and UI (including views)
//

#include <FL/fl_ask.H>

#include "impressionistDoc.h"
#include "impressionistUI.h"
#include "baseUI.h"

#include "ImpBrush.h"

#include "dotbrush.h"
// add when they are done, may be in different names, though.


ImpressionistDoc::ImpressionistDoc() {
	m_imageName[0]='\0';	// set NULL image name as init. 

	m_nWidth = -1;
	m_ucBitmap = NULL;
	m_ucPainting = NULL;

	// create one instance of each brush
	ImpBrush::c_nBrushCount = NUM_BRUSH_TYPE;
	ImpBrush::c_pBrushes = new ImpBrush* [ImpBrush::c_nBrushCount];

	ImpBrush::c_pBrushes[BRUSH_POINTS] = new DotBrush( this, "Points" );
// Note: You should implement these 5 brushes. They are set the same for a while
	ImpBrush::c_pBrushes[BRUSH_LINES] = new DotBrush( this, "Lines" );
	ImpBrush::c_pBrushes[BRUSH_CIRCLES] = new DotBrush( this, "Circles" );
	ImpBrush::c_pBrushes[BRUSH_SCATTERED_POINTS] = new DotBrush( this, "Scattered Points" );
	ImpBrush::c_pBrushes[BRUSH_SCATTERED_LINES] = new DotBrush( this, "Scattered Lines" );
	ImpBrush::c_pBrushes[BRUSH_SCATTERED_CIRCLES] = new DotBrush( this, "Scattered Circles" );

	// make one of the brushes current
	m_pCurrentBrush = ImpBrush::c_pBrushes[0];
}

//--------------------------------------------------------
// void ImpressionistDoc::setUI(ImpressionistUI* ui)
// set the current UI (created from FLUID)
//--------------------------------------------------------
void ImpressionistDoc::setUI(ImpressionistUI* ui) {
	m_pUI = ui;
}

//--------------------------------------------------------
// void ImpressionistDoc::getImageName
// return the active picture/painting name
//--------------------------------------------------------
char* ImpressionistDoc::getImageName() {
	return m_imageName;
}

//--------------------------------------------------------
// void ImpressionistDoc::setBrushType
// type : one of the defined brushes type
//		  called by the UI when the user change the brush type
//--------------------------------------------------------
void ImpressionistDoc::setBrushType(int type)
{
	m_pCurrentBrush = ImpBrush::c_pBrushes[type];
}

//--------------------------------------------------------
// void ImpressionistDoc::getSize
// return the size of the brush
//--------------------------------------------------------
int ImpressionistDoc::getSize()
{
	return m_pUI->getSize();
}

//--------------------------------------------------------
// void ImpressionistDoc::loadImage(char *iname)
// load the specified image
// this is called by the UI when the load image menu button is pressed
//--------------------------------------------------------

int ImpressionistDoc::loadImage(char *iname) {
	// try to open the image to read
	unsigned char *data;
	int width, height;

	if ( (data=readBMP(iname, width, height))==NULL ) {
		fl_alert("Can't load bitmap file");
		return 0;
	}

	// reflect the fact of loading the new image
	m_nPaintWidth = m_nWidth = width;
	m_nPaintHeight = m_nHeight = height;

	// release old storage
	if ( m_ucBitmap ) delete [] m_ucBitmap;
	if ( m_ucPainting ) delete [] m_ucPainting;

	m_ucBitmap = data;

	// allocate space for draw view
	m_ucPainting = new unsigned char [width*height*3];
	memset(m_ucPainting, 0, width*height*3);

	m_pUI->m_mainWindow->resize(m_pUI->m_mainWindow->x(), m_pUI->m_mainWindow->y(), width*2, height+25);

	// display it on origView
	m_pUI->m_origView->resizeWindow(width, height);	
	m_pUI->m_origView->refresh();

	// refresh paint view as well
	m_pUI->m_paintView->resizeWindow(width, height);	
	m_pUI->m_paintView->refresh();

	return 1;
}

//--------------------------------------------------------
// void ImpressionistDoc::saveImage(char *iname)
// save the specified image
// this is called by the UI when the save image menu button is pressed
//--------------------------------------------------------
int ImpressionistDoc::saveImage(char *iname) {

	writeBMP(iname, m_nPaintWidth, m_nPaintHeight, m_ucPainting);

	return 1;
}

//--------------------------------------------------------
// void ImpressionistDoc::GetOriginalPixel( int x, int y )
// get the color of the pixel in the original image at coord x and y
//--------------------------------------------------------
GLubyte* ImpressionistDoc::GetOriginalPixel( int x, int y )
{
	if ( x < 0 ) x = 0;
	else if ( x >= m_nWidth ) x = m_nWidth-1;
	if ( y < 0 ) y = 0;
	else if ( y >= m_nHeight ) y = m_nHeight-1;

	return (GLubyte*)(m_ucBitmap + 3 * (y*m_nWidth + x));
}

//--------------------------------------------------------
// void ImpressionistDoc::GetOriginalPixel( int x, int y )
// get the color of the pixel in the original image at point p
//--------------------------------------------------------
GLubyte* ImpressionistDoc::GetOriginalPixel( const Point p )
{
	return GetOriginalPixel( p.x, p.y );
}
